>From 2fefbe83fd5d0abed4aa063f5d0d2a7b59873bd8 Mon Sep 17 00:00:00 2001
From: David Edmondson <dme@dme.org>
Date: Thu, 11 Nov 2010 12:33:19 +0000
Subject: [PATCH] emacs: Improved printing support.

---
 emacs/Makefile.local   |    3 +-
 emacs/notmuch-print.el |   67 ++++++++++++++++++++++++++++++++++++++++++++++++
 emacs/notmuch-show.el  |   38 +++++++++++++++++++++++++++
 3 files changed, 107 insertions(+), 1 deletions(-)
 create mode 100644 emacs/notmuch-print.el

diff --git a/emacs/Makefile.local b/emacs/Makefile.local
index c5280f1..df87427 100644
--- a/emacs/Makefile.local
+++ b/emacs/Makefile.local
@@ -13,7 +13,8 @@ emacs_sources := \
 	$(dir)/notmuch-maildir-fcc.el \
 	$(dir)/notmuch-message.el \
 	$(dir)/coolj.el \
-	$(dir)/notmuch-lkml.el
+	$(dir)/notmuch-lkml.el \
+	$(dir)/notmuch-print.el
 
 emacs_images := \
 	$(dir)/notmuch-logo.png
diff --git a/emacs/notmuch-print.el b/emacs/notmuch-print.el
new file mode 100644
index 0000000..2ad4b45
--- /dev/null
+++ b/emacs/notmuch-print.el
@@ -0,0 +1,67 @@
+;; notmuch-print.el --- printing messages from notmuch.
+;;
+;; Copyright © David Edmondson
+;;
+;; This file is part of Notmuch.
+;;
+;; Notmuch is free software: you can redistribute it and/or modify it
+;; under the terms of the GNU General Public License as published by
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
+;;
+;; Notmuch is distributed in the hope that it will be useful, but
+;; WITHOUT ANY WARRANTY; without even the implied warranty of
+;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+;; General Public License for more details.
+;;
+;; You should have received a copy of the GNU General Public License
+;; along with Notmuch.  If not, see <http://www.gnu.org/licenses/>.
+;;
+;; Authors: David Edmondson <dme@dme.org>
+
+(defcustom notmuch-print-mechanism 'lpr
+  "How should printing be done?"
+  :group 'notmuch
+  :type '(choice
+	  (const :tag "Use lpr" lpr)
+	  (const :tag "Use ps-print" ps-print)
+	  (const :tag "Use muttprint" muttprint)
+	  (function :tag "Using a custom function")))
+
+(defvar notmuch-print-mechanism-map
+  '((lpr . notmuch-print-lpr)
+    (ps-print . notmuch-print-ps-print)
+    (muttprint . notmuch-print-muttprint)))
+
+(defmacro notmuch-print-with-file (filename &rest body)
+  `(with-temp-buffer
+     (insert-file-contents ,filename)
+     (setq buffer-modified-p nil)
+     ,@body))
+
+(defun notmuch-print-lpr (filename)
+  (notmuch-print-with-file filename
+			   (lpr-buffer)))
+
+(defun notmuch-print-ps-print (filename)
+  (notmuch-print-with-file filename
+			   (ps-print-buffer)))
+
+(defun notmuch-print-muttprint (filename)
+  (shell-command
+   (concat "cat " (shell-quote-argument filename) " | muttprint")))
+
+(defun notmuch-print-message (filename)
+  (let ((fn
+	 (cond
+	  ((functionp 'notmuch-print-mechanism)
+	   'notmuch-print-mechanism)
+	  ((assoc notmuch-print-mechanism notmuch-print-mechanism-map)
+	   (cdr (assoc notmuch-print-mechanism notmuch-print-mechanism-map)))
+	  (t
+	   (error "No printing mechanism found")))))
+    (funcall fn filename)))
+
+;;
+
+(provide 'notmuch-print)
diff --git a/emacs/notmuch-show.el b/emacs/notmuch-show.el
index 5efdf84..3680faf 100644
--- a/emacs/notmuch-show.el
+++ b/emacs/notmuch-show.el
@@ -32,6 +32,7 @@
 (require 'notmuch-query)
 (require 'notmuch-wash)
 (require 'notmuch-mua)
+(require 'notmuch-print)
 
 (declare-function notmuch-call-notmuch-process "notmuch" (&rest args))
 (declare-function notmuch-fontify-headers "notmuch" nil)
@@ -174,6 +175,38 @@ Set `notmuch-show-address-simplication' to:
       mm-handle (> (notmuch-count-attachments mm-handle) 1))))
   (message "Done"))
 
+(defun notmuch-show-with-message-as-text (fn)
+  "Apply function `fn' to a text representation of the current
+message."
+  (let* ((from (notmuch-show-clean-address
+		(mail-header-parse-address (notmuch-show-get-from))))
+	 (subject (notmuch-show-get-subject))
+	 (date (notmuch-show-get-date))
+	 (header (concat
+		  "Tags: " (mapconcat '(lambda (tag) tag)
+				      (notmuch-show-get-tags) ", ")
+		  "\n"
+		  "From: " from "\n"
+		  "Date: " date "\n"))
+	 (body (save-excursion
+		 (goto-char (notmuch-show-message-top))
+		 (forward-line)
+		 (buffer-substring (point) (notmuch-show-message-bottom))))
+	 (depth (notmuch-show-get-depth))
+	 (file (make-temp-file "notmuch")))
+    (with-temp-file file
+      (insert body)
+      (indent-rigidly (point-min) (point-max) (- depth))
+      (goto-char (point-min))
+      (insert header))
+    (funcall fn file)
+    (delete-file file)))
+
+(defun notmuch-show-print-message ()
+  "Print the current message."
+  (interactive)
+  (notmuch-show-with-message-as-text 'notmuch-print-message))
+
 (defun notmuch-show-fontify-header ()
   (let ((face (cond
 	       ((looking-at "[Tt]o:")
@@ -520,6 +553,8 @@ current buffer, if possible."
     (plist-put msg :message-invis-spec message-invis-spec)
     (overlay-put (make-overlay body-start body-end) 'invisible message-invis-spec)
 
+    (plist-put msg :depth depth)
+
     ;; Save the properties for this message. Currently this saves the
     ;; entire message (augmented it with other stuff), which seems
     ;; like overkill. We might save a reduced subset (for example, not
@@ -821,6 +856,9 @@ All currently available key bindings:
 (defun notmuch-show-get-to ()
   (notmuch-show-get-header :To))
 
+(defun notmuch-show-get-depth ()
+  (notmuch-show-get-prop :depth))
+
 (defun notmuch-show-set-tags (tags)
   "Set the tags of the current message."
   (notmuch-show-set-prop :tags tags)
-- 
1.7.2.3

